@@ -145,7 +145,7 @@ def load_esgf_pyclient_config():
             # List of available index nodes: https://esgf.llnl.gov/nodes.html
             # Be careful about the url, not all search urls have CMIP3 data?
             'urls': [
-                'https://esgf-index1.ceda.ac.uk/esg-search',
+                'https://esgf.ceda.ac.uk/esg-search',
                 'https://esgf-node.llnl.gov/esg-search',
                 'https://esgf-data.dkrz.de/esg-search',
                 'https://esgf-node.ipsl.upmc.fr/esg-search',
