@@ -1,6 +1,7 @@
 """Fixes for CESM2-FV2 model."""
 from .cesm2 import Cl as BaseCl
 from .cesm2 import Fgco2 as BaseFgco2
+from .cesm2 import Omon as BaseOmon
 from .cesm2 import Tas as BaseTas
 from ..common import SiconcFixScalarCoord
 
@@ -17,6 +18,9 @@ Clw = Cl
 Fgco2 = BaseFgco2
 
 
+Omon = BaseOmon
+
+
 Siconc = SiconcFixScalarCoord
 
 
