@@ -3,6 +3,7 @@ from .cnrm_cm6_1 import Cl as BaseCl
 from .cnrm_cm6_1 import Clcalipso as BaseClcalipso
 from .cnrm_cm6_1 import Cli as BaseCli
 from .cnrm_cm6_1 import Clw as BaseClw
+from .cnrm_cm6_1 import Omon as BaseOmon
 
 
 Cl = BaseCl
@@ -15,3 +16,6 @@ Cli = BaseCli
 
 
 Clw = BaseClw
+
+
+Omon = BaseOmon
