@@ -3,8 +3,16 @@ import cftime
 import dask.array as da
 import numpy as np
 
+from esmvalcore.iris_helpers import date2num
+
+from ..common import OceanFixGrid
 from ..fix import Fix
 
+Tos = OceanFixGrid
+
+
+Omon = OceanFixGrid
+
 
 class AllVars(Fix):
     """Fixes for all vars."""
@@ -24,6 +32,12 @@ class AllVars(Fix):
         """
         for cube in cubes:
             if cube.attributes['table_id'] == 'Amon':
+                for coord in ['latitude', 'longitude']:
+                    cube_coord = cube.coord(coord)
+                    bounds = cube_coord.bounds
+                    if np.any(bounds[:-1, 1] != bounds[1:, 0]):
+                        cube_coord.bounds = None
+                        cube_coord.guess_bounds()
                 time = cube.coord('time')
                 if np.any(time.bounds[:-1, 1] != time.bounds[1:, 0]):
                     times = time.units.num2date(time.points)
@@ -36,8 +50,8 @@ class AllVars(Fix):
                                               1, 1) if c.month < 12 else
                         cftime.DatetimeNoLeap(c.year + 1, 1, 1) for c in times
                     ]
-                    time.bounds = time.units.date2num(
-                        np.stack([starts, ends], -1))
+                    time.bounds = date2num(np.stack([starts, ends], -1),
+                                           time.units)
         return cubes
 
 
