@@ -4,7 +4,6 @@ import importlib
 import logging
 import os
 import re
-import warnings
 from copy import deepcopy
 from decimal import Decimal
 from typing import Dict
@@ -17,8 +16,6 @@ from geopy.geocoders import Nominatim
 from iris.analysis import AreaWeighted, Linear, Nearest, UnstructuredNearest
 from iris.util import broadcast_to_shape
 
-from esmvalcore.exceptions import ESMValCoreDeprecationWarning
-
 from ..cmor._fixes.shared import add_altitude_from_plev, add_plev_from_altitude
 from ..cmor.fix import fix_file, fix_metadata
 from ..cmor.table import CMOR_TABLES
@@ -387,6 +384,11 @@ def extract_point(cube, latitude, longitude, scheme):
     scalar, the dimension will be missing in the output cube (that is,
     it will be a scalar).
 
+    If the point to be extracted has at least one of the coordinate point
+    values outside the interval of the cube's same coordinate values, then
+    no extrapolation will be performed, and the resulting extracted cube
+    will have fully masked data.
+
     Parameters
     ----------
     cube : cube
@@ -400,8 +402,11 @@ def extract_point(cube, latitude, longitude, scheme):
 
     Returns
     -------
-    Returns a cube with the extracted point(s), and with adjusted
-    latitude and longitude coordinates (see above).
+    :py:class:`~iris.cube.Cube`
+        Returns a cube with the extracted point(s), and with adjusted
+        latitude and longitude coordinates (see above). If desired point
+        outside values for at least one coordinate, this cube will have fully
+        masked data.
 
     Raises
     ------
@@ -639,6 +644,11 @@ def regrid(cube, target_grid, scheme, lat_offset=True, lon_offset=True):
                     "'%s': %s", original_dtype, cube.core_data().dtype,
                     str(exc))
             cube.data = da.ma.masked_equal(cube.core_data(), fill_value)
+    else:
+        # force target coordinates
+        for coord in ['latitude', 'longitude']:
+            cube.coord(coord).points = target_grid.coord(coord).points
+            cube.coord(coord).bounds = target_grid.coord(coord).bounds
 
     return cube
 
@@ -847,21 +857,6 @@ def parse_vertical_scheme(scheme):
     (str, str)
         A tuple containing the interpolation and extrapolation scheme.
     """
-    # Issue warning when deprecated schemes are used
-    deprecated_schemes = {
-        'linear_horizontal_extrapolate_vertical': 'linear_extrapolate',
-        'nearest_horizontal_extrapolate_vertical': 'nearest_extrapolate',
-    }
-    if scheme in deprecated_schemes:
-        new_scheme = deprecated_schemes[scheme]
-        deprecation_msg = (
-            f"The vertical regridding scheme ``{scheme}`` has been deprecated "
-            f"in ESMValCore version 2.5.0 and is scheduled for removal in "
-            f"version 2.7.0. It has been renamed to the identical scheme "
-            f"``{new_scheme}`` without any change in functionality.")
-        warnings.warn(deprecation_msg, ESMValCoreDeprecationWarning)
-        scheme = new_scheme
-
     # Check if valid scheme is given
     if scheme not in VERTICAL_SCHEMES:
         raise ValueError(
@@ -1086,3 +1081,43 @@ def get_reference_levels(filename, project, dataset, short_name, mip,
     except iris.exceptions.CoordinateNotFoundError:
         raise ValueError('z-coord not available in {}'.format(filename))
     return coord.points.tolist()
+
+
+def extract_coordinate_points(cube, definition, scheme):
+    """Extract points from any coordinate with interpolation.
+
+    Multiple points can also be extracted, by supplying an array of
+    coordinates. The resulting point cube will match the respective
+    coordinates to those of the input coordinates.
+    If the input coordinate is a scalar, the dimension will be a
+    scalar in the output cube.
+
+    Parameters
+    ----------
+    cube : cube
+        The source cube to extract a point from.
+    defintion : dict(str, float or array of float)
+        The coordinate - values pairs to extract
+    scheme : str
+        The interpolation scheme. 'linear' or 'nearest'. No default.
+
+    Returns
+    -------
+    :py:class:`~iris.cube.Cube`
+        Returns a cube with the extracted point(s), and with adjusted
+        latitude and longitude coordinates (see above). If desired point
+        outside values for at least one coordinate, this cube will have fully
+        masked data.
+
+    Raises
+    ------
+    ValueError:
+        If the interpolation scheme is not provided or is not recognised.
+    """
+
+    msg = f"Unknown interpolation scheme, got {scheme!r}."
+    scheme = POINT_INTERPOLATION_SCHEMES.get(scheme.lower())
+    if not scheme:
+        raise ValueError(msg)
+    cube = cube.interpolate(definition.items(), scheme=scheme)
+    return cube
