@@ -7,8 +7,19 @@ import iris
 import numpy as np
 import pytest
 
+from esmvalcore.cmor._fixes.cmip6.cesm2 import Cl as BaseCl
+from esmvalcore.cmor._fixes.cmip6.cesm2 import Fgco2 as BaseFgco2
 from esmvalcore.cmor._fixes.cmip6.cesm2 import Tas as BaseTas
-from esmvalcore.cmor._fixes.cmip6.cesm2_waccm import Cl, Cli, Clw, Tas
+from esmvalcore.cmor._fixes.cmip6.cesm2_waccm import (
+    Cl,
+    Cli,
+    Clw,
+    Fgco2,
+    Omon,
+    Siconc,
+    Tas,
+)
+from esmvalcore.cmor._fixes.common import SiconcFixScalarCoord
 from esmvalcore.cmor.fix import Fix
 
 
@@ -18,6 +29,11 @@ def test_get_cl_fix():
     assert fix == [Cl(None)]
 
 
+def test_cl_fix():
+    """Test fix for ``cl``."""
+    assert issubclass(Cl, BaseCl)
+
+
 @pytest.mark.skipif(sys.version_info < (3, 7, 6),
                     reason="requires python3.7.6 or newer")
 @unittest.mock.patch(
@@ -66,6 +82,28 @@ def test_clw_fix():
     assert Clw is Cl
 
 
+def test_get_fgco2_fix():
+    """Test getting of fix."""
+    fix = Fix.get_fixes('CMIP6', 'CESM2-WACCM', 'Omon', 'fgco2')
+    assert fix == [Fgco2(None), Omon(None)]
+
+
+def test_fgco2_fix():
+    """Test fix for ``fgco2``."""
+    assert Fgco2 is BaseFgco2
+
+
+def test_get_siconc_fix():
+    """Test getting of fix."""
+    fix = Fix.get_fixes('CMIP6', 'CESM2-WACCM', 'SImon', 'siconc')
+    assert fix == [Siconc(None)]
+
+
+def test_siconc_fix():
+    """Test fix for ``siconc``."""
+    assert Siconc is SiconcFixScalarCoord
+
+
 @pytest.fixture
 def tas_cubes():
     """Cubes to test fixes for ``tas``."""
