@@ -7,10 +7,13 @@ from unittest.mock import call
 import numpy as np
 import pytest
 from cf_units import Unit
-from iris.aux_factory import HybridHeightFactory, HybridPressureFactory
+from iris.aux_factory import (
+    AtmosphereSigmaFactory,
+    HybridHeightFactory,
+    HybridPressureFactory,
+)
 from iris.coords import AuxCoord, DimCoord
 from iris.cube import Cube, CubeList
-from iris.exceptions import ConcatenateError
 
 from esmvalcore.preprocessor import _io
 
@@ -65,6 +68,31 @@ def mock_empty_cube():
     return cube
 
 
+@pytest.fixture
+def mock_atmosphere_sigma_cube():
+    """Return mocked cube with atmosphere sigma coordinate."""
+    cube = unittest.mock.create_autospec(Cube, spec_set=True, instance=True)
+    ptop_coord = AuxCoord([1.0], var_name='ptop', units='Pa')
+    lev_coord = AuxCoord([0.0], bounds=[[-0.5, 1.5]], var_name='lev',
+                         units='1')
+    ps_coord = AuxCoord([[[100000]]], var_name='ps', units='Pa')
+    cube.coord.side_effect = [ptop_coord, lev_coord, ps_coord,
+                              ptop_coord, lev_coord, ps_coord]
+    cube.coords.return_value = [
+        ptop_coord,
+        lev_coord,
+        ps_coord,
+        AuxCoord(0.0, standard_name='atmosphere_sigma_coordinate'),
+    ]
+    aux_factory = AtmosphereSigmaFactory(
+        pressure_at_top=ptop_coord,
+        sigma=lev_coord,
+        surface_air_pressure=ps_coord,
+    )
+    cube.aux_factories = ['dummy', aux_factory]
+    return cube
+
+
 @pytest.fixture
 def mock_hybrid_height_cube():
     """Return mocked cube with hybrid height coordinate."""
@@ -147,6 +175,29 @@ def test_fix_aux_factories_empty_cube(mock_empty_cube):
     assert mock_empty_cube.mock_calls == [call.coords()]
 
 
+def test_fix_aux_factories_atmosphere_sigma(mock_atmosphere_sigma_cube):
+    """Test fixing of atmosphere sigma coordinate."""
+    check_if_fix_aux_factories_is_necessary()
+
+    # Test with aux_factory object
+    _io._fix_aux_factories(mock_atmosphere_sigma_cube)
+    mock_atmosphere_sigma_cube.coords.assert_called_once_with()
+    mock_atmosphere_sigma_cube.coord.assert_has_calls([call(var_name='ptop'),
+                                                       call(var_name='lev'),
+                                                       call(var_name='ps')])
+    mock_atmosphere_sigma_cube.add_aux_factory.assert_not_called()
+
+    # Test without aux_factory object
+    mock_atmosphere_sigma_cube.reset_mock()
+    mock_atmosphere_sigma_cube.aux_factories = ['dummy']
+    _io._fix_aux_factories(mock_atmosphere_sigma_cube)
+    mock_atmosphere_sigma_cube.coords.assert_called_once_with()
+    mock_atmosphere_sigma_cube.coord.assert_has_calls([call(var_name='ptop'),
+                                                       call(var_name='lev'),
+                                                       call(var_name='ps')])
+    mock_atmosphere_sigma_cube.add_aux_factory.assert_called_once()
+
+
 def test_fix_aux_factories_hybrid_height(mock_hybrid_height_cube):
     """Test fixing of hybrid height coordinate."""
     check_if_fix_aux_factories_is_necessary()
@@ -323,8 +374,10 @@ class TestConcatenate(unittest.TestCase):
                      var_name='sample',
                      dim_coords_and_dims=((time_coord_2, 0), ))
         cubes_single_ovlp = [cube2, cube1]
-        with self.assertRaises(ConcatenateError):
-            _io.concatenate(cubes_single_ovlp)
+        cubess = _io.concatenate(cubes_single_ovlp)
+        # this tests the scalar to vector cube conversion too
+        time_points = cubess.coord("time").core_points()
+        np.testing.assert_array_equal(time_points, [1., 1.5, 5., 7.])
 
     def test_concatenate_no_time_coords(self):
         """Test a more generic case."""
@@ -384,7 +437,7 @@ class TestConcatenate(unittest.TestCase):
             Cube([33., 55.],
                  var_name='sample',
                  dim_coords_and_dims=((time_coord, 0), )))
-        with self.assertRaises(TypeError):
+        with self.assertRaises((TypeError, ValueError)):
             _io.concatenate(self.raw_cubes)
 
     def test_fail_on_units_concatenate_with_overlap(self):
